@@ -2115,7 +2115,7 @@ int receive_run_reply(struct dlm_header *hd, int len)
 int receive_run_request(struct dlm_header *hd, int len)
 {
 	struct run_request *req = (struct run_request *)hd;
-	struct run *run;
+	struct run *run = NULL;
 
 	run_request_in(req);
 
@@ -2153,7 +2153,7 @@ int receive_run_request(struct dlm_header *hd, int len)
 		return 0;
 	}
 
-	if (!opt(enable_helper_ind)) {
+	if (!opt(enable_helper_ind) && run) {
 		log_debug("receive_run_request %s helper not enabled", req->uuid);
 		run->info.reply_count++;
 		run->info.need_replies--;
