@@ -81,6 +81,17 @@ static int do_connect(const char *sock_path)
 	return fd;
 }
 
+static inline void init_header_name(struct dlmc_header *h,
+				    const char *name, size_t len)
+{
+#pragma GCC diagnostic push
+#if __GNUC__ >= 8
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#endif
+	strncpy(h->name, name, len);
+#pragma GCC diagnostic pop
+}
+
 static void init_header(struct dlmc_header *h, int cmd, char *name,
 			int extra_len)
 {
@@ -92,7 +103,7 @@ static void init_header(struct dlmc_header *h, int cmd, char *name,
 	h->command = cmd;
 
 	if (name)
-		strncpy(h->name, name, DLM_LOCKSPACE_LEN);
+		init_header_name(h, name, DLM_LOCKSPACE_LEN);
 }
 
 static char copy_buf[DLMC_DUMP_SIZE];
@@ -881,7 +892,7 @@ int dlmc_run_check(char *run_uuid, int len, int wait_sec, uint32_t flags,
 
 	init_header(&h, DLMC_CMD_RUN_CHECK, NULL, 0);
 	h.flags = flags;
-	strncpy(h.name, run_uuid, DLMC_RUN_UUID_LEN);
+	init_header_name(&h, run_uuid, DLMC_RUN_UUID_LEN);
 
 	memset(&rh, 0, sizeof(rh));
 
