@@ -13,6 +13,7 @@
 #include <linux/netlink.h>
 #include <linux/genetlink.h>
 #include <linux/dlm_netlink.h>
+#include <uuid/uuid.h>
 
 #ifdef USE_SD_NOTIFY
 #include <systemd/sd-daemon.h>
@@ -31,6 +32,13 @@ static pthread_mutex_t query_mutex;
 static struct list_head fs_register_list;
 static int kernel_monitor_fd;
 
+int helper_ci;
+int helper_pid;
+int helper_req_fd;
+int helper_status_fd;
+uint64_t helper_last_status;
+uint32_t helper_full_count;
+
 struct client {
 	int fd;
 	void *workfn;
@@ -172,6 +180,363 @@ static void sigchld_handler(int sig)
 {
 }
 
+struct run *find_run(char *uuid_str)
+{
+	struct run *run;
+
+	list_for_each_entry(run, &run_ops, list) {
+		if (!strcmp(run->uuid, uuid_str))
+			return run;
+	}
+	return NULL;
+}
+
+static void close_helper(void)
+{
+	close(helper_req_fd);
+	close(helper_status_fd);
+	helper_req_fd = -1;
+	helper_status_fd = -1;
+	pollfd[helper_ci].fd = -1;
+	pollfd[helper_ci].events = 0;
+	helper_ci = -1;
+
+	/* don't set helper_pid = -1 until we've tried waitpid */
+}
+
+/*
+ * We cannot block the main thread on this write, so the pipe is NONBLOCK, and
+ * write fails with EAGAIN when the pipe is full.  With around 1.5K request
+ * size and 64k default pipe size, the pipe will be full if we quickly send
+ * around 40 requests to the helper.  We retry the message once a second, so
+ * we'll retry the write again in a second.
+ *
+ * By setting the pipe size to 1MB in setup_helper, we could quickly send many
+ * more requests before getting EAGAIN.
+ */
+
+void send_helper_run_request(struct run_request *req)
+{
+	int rv;
+
+	if (helper_req_fd == -1) {
+		log_error("send_helper_run_request no fd");
+		return;
+	}
+
+ retry:
+	rv = write(helper_req_fd, req, sizeof(struct run_request));
+	if (rv == -1 && errno == EINTR)
+		goto retry;
+
+	/* pipe is full, we'll try again in a second */
+	if (rv == -1 && errno == EAGAIN) {
+		helper_full_count++;
+		log_debug("send_helper_run_request full_count %u",
+			  helper_full_count);
+		return;
+	}
+
+	/* helper exited or closed fd, quit using helper */
+	if (rv == -1 && errno == EPIPE) {
+		log_error("send_helper_run_request EPIPE");
+		close_helper();
+		return;
+	}
+
+	if (rv != sizeof(struct run_request)) {
+		/* this shouldn't happen */
+		log_error("send_helper_run_request %d %d", rv, errno);
+		close_helper();
+		return;
+	}
+}
+
+static void send_helper_run_cancel(struct run *run)
+{
+	struct run_request req;
+	int rv;
+
+	if (helper_req_fd == -1) {
+		log_error("send_helper_run_cancel no fd");
+		return;
+	}
+
+	memset(&req, 0, sizeof(req));
+	memcpy(req.uuid, run->uuid, RUN_UUID_LEN);
+
+	rv = write(helper_req_fd, &req, sizeof(struct run_request));
+	if (rv < 0)
+		log_error("send_helper_run_cancel write error");
+}
+
+/*
+ * first pipe for daemon to send requests to helper; they are not acknowledged
+ * and the daemon does not get any result back for the requests.
+ *
+ * second pipe for helper to send general status/heartbeat back to the daemon
+ * every so often to confirm it's not dead/hung.  If the helper gets stuck or
+ * killed, the daemon will not get the status and won't bother sending requests
+ * to the helper, and use SIGTERM instead
+ */
+
+static int setup_helper(void)
+{
+	int pid;
+	int pw_fd = -1; /* parent write */
+	int cr_fd = -1; /* child read */
+	int pr_fd = -1; /* parent read */
+	int cw_fd = -1; /* child write */
+	int pfd[2];
+
+	/* we can't allow the main daemon thread to block */
+	if (pipe2(pfd, O_NONBLOCK | O_CLOEXEC))
+		return -errno;
+
+	/* uncomment for rhel7 where this should be available */
+	/* fcntl(pfd[1], F_SETPIPE_SZ, 1024*1024); */
+
+	cr_fd = pfd[0];
+	pw_fd = pfd[1];
+
+	if (pipe2(pfd, O_NONBLOCK | O_CLOEXEC)) {
+		close(cr_fd);
+		close(pw_fd);
+		return -errno;
+	}
+
+	pr_fd = pfd[0];
+	cw_fd = pfd[1];
+
+	pid = fork();
+	if (pid < 0) {
+		close(cr_fd);
+		close(pw_fd);
+		close(pr_fd);
+		close(cw_fd);
+		return -errno;
+	}
+
+	if (pid) {
+		close(cr_fd);
+		close(cw_fd);
+		helper_req_fd = pw_fd;
+		helper_status_fd = pr_fd;
+		helper_pid = pid;
+		return 0;
+	} else {
+		close(pr_fd);
+		close(pw_fd);
+		run_helper(cr_fd, cw_fd, opt(daemon_debug_ind));
+		exit(0);
+	}
+}
+
+static void process_helper(int ci)
+{
+	struct run_reply reply;
+	struct run_reply send_reply;
+	struct run *run;
+	int rv;
+
+	rv = read(client[ci].fd, &reply, sizeof(reply));
+	if (!rv || rv == -EAGAIN)
+		return;
+	if (rv < 0) {
+		log_error("process_helper rv %d errno %d", rv, errno);
+		goto fail;
+	}
+	if (rv != sizeof(reply)) {
+		log_error("process_helper recv size %d", rv);
+		goto fail;
+	}
+
+	if (!reply.header.type) {
+		/* log_debug("helper status"); */
+		helper_last_status = monotime();
+		return;
+	}
+
+	if (reply.header.type == DLM_MSG_RUN_REPLY) {
+		run = find_run(reply.uuid);
+		if (!run) {
+			log_error("helper reply no run uuid %s", reply.uuid);
+			return;
+		}
+
+		memset(&send_reply, 0, sizeof(send_reply));
+		memcpy(&send_reply.info, &run->info, sizeof(struct run_info));
+		memcpy(send_reply.uuid, run->uuid, RUN_UUID_LEN);
+		send_reply.header.type = DLM_MSG_RUN_REPLY;
+		send_reply.info.local_pid = reply.info.local_pid;
+		send_reply.info.local_result = reply.info.local_result;
+
+		log_debug("helper reply %s pid %d result %d",
+			  send_reply.uuid, send_reply.info.local_pid, send_reply.info.local_result);
+
+		send_run_reply(run, &send_reply);
+		return;
+	}
+
+	return;
+
+ fail:
+	close_helper();
+}
+
+static void helper_dead(int ci)
+{
+	int pid = helper_pid;
+	int rv, status;
+
+	close_helper();
+
+	helper_pid = -1;
+
+	rv = waitpid(pid, &status, WNOHANG);
+
+	if (rv != pid) {
+		/* should not happen */
+		log_error("helper pid %d dead wait %d", pid, rv);
+		return;
+	}
+
+	if (WIFEXITED(status)) {
+		log_error("helper pid %d exit status %d", pid,
+			  WEXITSTATUS(status));
+		return;
+	}
+
+	if (WIFSIGNALED(status)) {
+		log_error("helper pid %d term signal %d", pid,
+			  WTERMSIG(status));
+		return;
+	}
+
+	/* should not happen */
+	log_error("helper pid %d state change", pid);
+}
+
+static int start_run_operation(char *data, int datalen, int dest_nodeid, uint32_t flags, char *uuid_out)
+{
+	struct run *run;
+	struct run_request req;
+	uuid_t uu;
+	int rv;
+
+	if (!opt(enable_helper_ind)) {
+		log_debug("ignore start_run helper not enabled");
+		return -1;
+	}
+
+	if (datalen > RUN_COMMAND_LEN)
+		return -1;
+
+	if (!(run = malloc(sizeof(struct run))))
+		return -1;
+
+	memset(run, 0, sizeof(struct run));
+
+	uuid_generate(uu);
+	uuid_unparse_lower(uu, run->uuid);
+	strncpy(run->command, data, datalen);
+	run->info.start_nodeid = our_nodeid;
+	run->info.dest_nodeid = dest_nodeid;
+	run->info.flags = flags;
+
+	memset(&req, 0, sizeof(req));
+	req.header.type = DLM_MSG_RUN_REQUEST;
+	memcpy(&req.info, &run->info, sizeof(struct run_info));
+	memcpy(req.uuid, run->uuid, RUN_UUID_LEN);
+	strncpy(req.command, data, datalen);
+
+	log_error("run start %s %.128s", run->uuid, run->command);
+
+	rv = send_run_request(run, &req);
+
+	memcpy(uuid_out, run->uuid, RUN_UUID_LEN);
+	list_add(&run->list, &run_ops);
+
+	/*
+	 * This flag means the starting node should run the command itself
+	 * at the time of the request and not wait to receive its own request.
+	 */
+	if (flags & DLMC_FLAG_RUN_START_NODE_FIRST)
+		send_helper_run_request(&req);
+
+	return rv;
+}
+
+void clear_run(struct run *run)
+{
+	log_debug("clear run %s", run->uuid);
+	list_del(&run->list);
+	free(run);
+}
+
+static int check_run_operation(char *uuid_str, uint32_t flags, struct dlmc_run_check_state *state)
+{
+	char nodes_buf[128];
+	struct run *run;
+	int pos, len, ret, i;
+
+	if (!opt(enable_helper_ind)) {
+		log_debug("ignore check_run helper not enabled");
+		return -1;
+	}
+
+	run = find_run(uuid_str);
+	if (!run) {
+		log_debug("check_run no uuid %s", uuid_str);
+		return -1;
+	}
+
+	if (flags & DLMC_FLAG_RUN_CHECK_CANCEL) {
+		log_debug("cancel_run %s", run->uuid);
+		send_helper_run_cancel(run);
+		clear_run(run);
+		return 0;
+	}
+
+	log_debug("check_run %s reply_count %d need_replies %d fail_count %d",
+		  uuid_str, run->info.reply_count, run->info.need_replies, run->info.fail_count);
+
+	if (run->info.need_replies)
+		state->check_status |= DLMC_RUN_STATUS_WAITING;
+	else
+		state->check_status |= DLMC_RUN_STATUS_DONE;
+
+	if (run->info.fail_count)
+		state->check_status |= DLMC_RUN_STATUS_FAILED;
+
+	if (!run->info.need_replies) {
+		if (run->info.fail_count) {
+			/* create a printable list of nodeids where the command failed */
+			pos = 0;
+			len = sizeof(nodes_buf);
+			memset(nodes_buf, 0, len);
+			for (i = 0; i < run->node_count; i++) {
+				if (!run->node_results[i].result)
+					continue;
+				ret = snprintf(nodes_buf + pos, len - pos, "%d ", run->node_results[i].nodeid);
+				if (ret >= len - pos)
+					break;
+				pos += ret;
+			}
+			nodes_buf[len-1] = '\0';
+
+			log_error("run ended %s error from %d remote nodes with ids: %s", run->uuid, run->info.fail_count, nodes_buf);
+		} else {
+			log_error("run ended %s success from %d remote nodes", run->uuid, run->info.reply_count);
+		}
+	}
+
+	if (!run->info.need_replies && (flags & DLMC_FLAG_RUN_CHECK_CLEAR))
+		clear_run(run);
+
+	return 0;
+}
+
 static struct lockspace *create_ls(char *name)
 {
 	struct lockspace *ls;
@@ -350,7 +715,10 @@ static void process_uevent(int ci)
 	act = argv[0];
 	sys = argv[2];
 
-	if ((strlen(sys) != strlen("dlm")) || strcmp(sys, "dlm"))
+	if (!act || !sys || !argv[3])
+		return;
+
+	if (strncmp(sys, "dlm", 3))
 		return;
 
 	log_debug("kernel: %s %s", act, argv[3]);
@@ -397,7 +765,7 @@ static void process_uevent(int ci)
 static int setup_uevent(void)
 {
 	struct sockaddr_nl snl;
-	int s, rv;
+	int s, rv, val;
 
 	s = socket(AF_NETLINK, SOCK_DGRAM, NETLINK_KOBJECT_UEVENT);
 	if (s < 0) {
@@ -405,6 +773,41 @@ static int setup_uevent(void)
 		return s;
 	}
 
+	/* man 7 netlink:
+	 *
+	 * However,  reliable  transmissions from kernel to user are impossible in
+	 * any case.  The kernel can't send a netlink message if the socket buffer
+	 * is  full: the message will be dropped and the kernel and the user-space
+	 * process will no longer have the same view of kernel state.  It is up to
+	 * the  application  to  detect  when  this happens (via the ENOBUFS error
+	 * returned by recvmsg(2)) and resynchronize.
+	 *
+	 * To avoid ENOBUFS errors we set the netlink socket to realiable
+	 * transmission mode which can be turned on by NETLINK_NO_ENOBUFS
+	 * option. This option is available since kernel 2.6.30. If this setting
+	 * fails we fallback to increase the netlink socket receive buffer.
+	 */
+	val = 1;
+	rv = setsockopt(s, SOL_NETLINK, NETLINK_NO_ENOBUFS, &val, sizeof(val));
+	if (rv == -1) {
+		/* Fallback handling if NETLINK_NO_ENOBUFS fails to set.
+		 *
+		 * To prevent ENOBUFS errors we just set the receive buffer to
+		 * two megabyte as other applications do it. This will not
+		 * ensure that we never receive ENOBUFS but it's more unlikely.
+		 */
+		val = DEFAULT_NETLINK_RCVBUF;
+		log_error("uevent netlink NETLINK_NO_ENOBUFS errno %d, will set rcvbuf to %d bytes", errno, val);
+
+		rv = setsockopt(s, SOL_SOCKET, SO_RCVBUF, &val, sizeof(val));
+		if (rv == -1)
+			log_error("uevent netlink SO_RCVBUF errno %d", errno);
+
+		rv = setsockopt(s, SOL_SOCKET, SO_RCVBUFFORCE, &val, sizeof(val));
+		if (rv == -1)
+			log_error("uevent netlink SO_RCVBUFFORCE errno %d", errno);
+	}
+
 	memset(&snl, 0, sizeof(snl));
 	snl.nl_family = AF_NETLINK;
 	snl.nl_pid = getpid();
@@ -420,6 +823,17 @@ static int setup_uevent(void)
 	return s;
 }
 
+static inline void init_header_name(struct dlmc_header *h,
+				    const char *name, size_t len)
+{
+#pragma GCC diagnostic push
+#if __GNUC__ >= 8
+#pragma GCC diagnostic ignored "-Wstringop-truncation"
+#endif
+	strncpy(h->name, name, len);
+#pragma GCC diagnostic pop
+}
+
 static void init_header(struct dlmc_header *h, int cmd, char *name, int result,
 			int extra_len)
 {
@@ -432,11 +846,61 @@ static void init_header(struct dlmc_header *h, int cmd, char *name, int result,
 	h->data = result;
 
 	if (name)
-		strncpy(h->name, name, DLM_LOCKSPACE_LEN);
+		init_header_name(h, name, DLM_LOCKSPACE_LEN);
 }
 
 static char copy_buf[LOG_DUMP_SIZE];
 
+static void copy_run_list(char *buf, int *len)
+{
+	char tmp[1024];
+	struct run *run;
+	int ret, pos = 0;
+
+	list_for_each_entry(run, &run_ops, list) {
+		memset(tmp, 0, sizeof(tmp));
+
+		snprintf(tmp, 1024, "run_uuid %s start_nodeid %d local_pid %d local_result %d need_replies %d reply_count %d fail_count %d flags %x\n",
+			 run->uuid, run->info.start_nodeid, run->info.local_pid,
+			 run->info.local_result, run->info.need_replies,
+			 run->info.reply_count, run->info.fail_count, run->info.flags);
+
+		if (pos + strlen(tmp) >= LOG_DUMP_SIZE)
+			break;
+
+		ret = sprintf(buf + pos, "%s", tmp);
+		pos += ret;
+
+		memset(tmp, 0, sizeof(tmp));
+
+		snprintf(tmp, 1024, "run_command %.1000s\n", run->command);
+
+		if (pos + strlen(tmp) >= LOG_DUMP_SIZE)
+			break;
+
+		ret = sprintf(buf + pos, "%s", tmp);
+		pos += ret;
+
+		/* TODO: dump node results */
+	}
+
+	*len = pos;
+}
+
+static void query_dump_run(int fd)
+{
+	struct dlmc_header h;
+	int len = 0;
+
+	copy_run_list(copy_buf, &len);
+
+	init_header(&h, DLMC_CMD_DUMP_RUN, NULL, 0, len);
+	send(fd, &h, sizeof(h), MSG_NOSIGNAL);
+
+	if (len)
+		send(fd, copy_buf, len, MSG_NOSIGNAL);
+}
+
 static void query_dump_debug(int fd)
 {
 	struct dlmc_header h;
@@ -664,9 +1128,13 @@ static void query_lockspace_nodes(int fd, char *name, int option, int max)
 static void process_connection(int ci)
 {
 	struct dlmc_header h;
+	char uuid_str[RUN_UUID_LEN];
 	char *extra = NULL;
-	int rv, extra_len;
+	int rv, extra_len = 0;
 	struct lockspace *ls;
+	struct dlmc_run_check_state state;
+
+	memset(uuid_str, 0, sizeof(uuid_str));
 
 	rv = do_read(client[ci].fd, &h, sizeof(h));
 	if (rv < 0) {
@@ -736,6 +1204,24 @@ static void process_connection(int ci)
 			 h.data, NULL, 0);
 		break;
 
+	case DLMC_CMD_RUN_START:
+		if (!extra_len)
+			rv = -EINVAL;
+		else
+			rv = start_run_operation(extra, extra_len, h.data, h.flags, uuid_str);
+		do_reply(client[ci].fd, DLMC_CMD_RUN_START, uuid_str, rv, 0, NULL, 0);
+		client_dead(ci);
+		break;
+
+	case DLMC_CMD_RUN_CHECK:
+		memset(&state, 0, sizeof(state));
+
+		rv = check_run_operation(h.name, h.flags, &state);
+
+		do_reply(client[ci].fd, DLMC_CMD_RUN_CHECK, NULL, rv, 0, (char *)&state, sizeof(state));
+		/* dlmc_run_check may retry checks on the same connection */
+		break;
+
 #if 0
 	case DLMC_CMD_DEADLOCK_CHECK:
 		ls = find_ls(h.name);
@@ -879,6 +1365,9 @@ static void *process_queries(void *arg)
 			send_state_daemon_nodes(f);
 			send_state_startup_nodes(f);
 			break;
+		case DLMC_CMD_DUMP_RUN:
+			query_dump_run(f);
+			break;
 		default:
 			break;
 		}
@@ -1017,6 +1506,9 @@ static int loop(void)
 	plock_fd = rv;
 	plock_ci = client_add(rv, process_plocks, NULL);
 
+	if (opt(enable_helper_ind))
+		helper_ci = client_add(helper_status_fd, process_helper, helper_dead);
+
 #ifdef USE_SD_NOTIFY
 	sd_notify(0, "READY=1");
 #endif
@@ -1232,6 +1724,8 @@ static void print_usage(void)
 			printf(" [%d]\n", o->default_int);
 		else if (o->req_arg == req_arg_bool)
 			printf(" [%d]\n", o->default_int);
+		else if (o->req_arg == req_arg_uint)
+			printf(" [%u]\n", o->default_uint);
 		else if (o->req_arg == no_arg && !o->default_int)
 			printf(" [0]\n");
 		else
@@ -1242,7 +1736,8 @@ static void print_usage(void)
 }
 
 static void set_opt_default(int ind, const char *name, char letter, int arg_type,
-			    int default_int, const char *default_str, const char *desc)
+			    int default_int, const char *default_str,
+			    unsigned int default_uint, const char *desc)
 {
 	dlm_options[ind].name = name;
 	dlm_options[ind].letter = letter;
@@ -1250,130 +1745,147 @@ static void set_opt_default(int ind, const char *name, char letter, int arg_type
 	dlm_options[ind].desc = desc;
 	dlm_options[ind].default_int = default_int;
 	dlm_options[ind].default_str = default_str;
+	dlm_options[ind].default_uint = default_uint;
 	dlm_options[ind].use_int = default_int;
 	dlm_options[ind].use_str = (char *)default_str;
+	dlm_options[ind].use_uint = default_uint;
 }
 
 static void set_opt_defaults(void)
 {
 	set_opt_default(daemon_debug_ind,
 			"daemon_debug", 'D', no_arg,
-			0, NULL,
+			0, NULL, 0,
 			"enable debugging to stderr and don't fork");
 
 	set_opt_default(foreground_ind,
 			"foreground", '\0', no_arg,
-			0, NULL,
+			0, NULL, 0,
 			"don't fork");
 
 	set_opt_default(log_debug_ind,
 			"log_debug", 'K', no_arg,
-			0, NULL,
+			0, NULL, 0,
 			"enable kernel dlm debugging messages");
 
 	set_opt_default(timewarn_ind,
 			"timewarn", '\0', req_arg_int,
-			0, NULL,
+			0, NULL, 0,
 			""); /* do not advertise */
 
 	set_opt_default(protocol_ind,
 			"protocol", 'r', req_arg_str,
-			-1, "detect",
+			-1, "detect", 0,
 			"dlm kernel lowcomms protocol: tcp, sctp, detect");
 
-	set_opt_default(enable_inaddr_any_ind,
-			"enable_inaddr_any", '\0', req_arg_int,
-			0, NULL,
-			"local node to use inaddr_any: 0=disable, 1=auto, 4=IPv4, 6=IPv6");
+	set_opt_default(port_ind,
+			"port", 'R', req_arg_uint,
+			-1, NULL, 21064,
+			"dlm kernel lowcomms protocol port");
+
+	set_opt_default(bind_all_ind,
+			"bind_all", '\0', req_arg_int,
+			0, NULL, 0,
+			""); /* do not advertise */
+
+	set_opt_default(mark_ind,
+			"mark", '\0', req_arg_uint,
+			0, NULL, 0,
+			"set mark value for DLM if not explicit by nodeid specified");
 
 	set_opt_default(debug_logfile_ind,
 			"debug_logfile", 'L', no_arg,
-			0, NULL,
+			0, NULL, 0,
 			"write debugging to log file");
 
 	set_opt_default(enable_fscontrol_ind,
 			"enable_fscontrol", '\0', req_arg_bool,
-			0, NULL,
+			0, NULL, 0,
 			""); /* do not advertise */
 
 	set_opt_default(enable_plock_ind,
 			"enable_plock", 'p', req_arg_bool,
-			1, NULL,
+			1, NULL, 0,
 			"enable/disable posix lock support for cluster fs");
 
 	set_opt_default(plock_debug_ind,
 			"plock_debug", 'P', no_arg,
-			0, NULL,
+			0, NULL, 0,
 			"enable plock debugging");
 
 	set_opt_default(plock_rate_limit_ind,
 			"plock_rate_limit", 'l', req_arg_int,
-			0, NULL,
+			0, NULL, 0,
 			"limit rate of plock operations (0 for none)");
 
 	set_opt_default(plock_ownership_ind,
 			"plock_ownership", 'o', req_arg_bool,
-			0, NULL,
+			0, NULL, 0,
 			"enable/disable plock ownership");
 
 	set_opt_default(drop_resources_time_ind,
 			"drop_resources_time", 't', req_arg_int,
-			10000, NULL,
+			10000, NULL, 0,
 			"plock ownership drop resources time (milliseconds)");
 
 	set_opt_default(drop_resources_count_ind,
 			"drop_resources_count", 'c', req_arg_int,
-			10, NULL,
+			10, NULL, 0,
 			"plock ownership drop resources count");
 
 	set_opt_default(drop_resources_age_ind,
 			"drop_resources_age", 'a', req_arg_int,
-			10000, NULL,
+			10000, NULL, 0,
 			"plock ownership drop resources age (milliseconds)");
 
 	set_opt_default(post_join_delay_ind,
 			"post_join_delay", 'j', req_arg_int,
-			30, NULL,
+			30, NULL, 0,
 			"seconds to delay fencing after cluster join");
 
 	set_opt_default(enable_fencing_ind,
 			"enable_fencing", 'f', req_arg_bool,
-			1, NULL,
+			1, NULL, 0,
 			"enable/disable fencing");
 
 	set_opt_default(enable_concurrent_fencing_ind,
 			"enable_concurrent_fencing", '\0', req_arg_bool,
-			0, NULL,
+			0, NULL, 0,
 			"enable/disable concurrent fencing");
 
 	set_opt_default(enable_startup_fencing_ind,
 			"enable_startup_fencing", 's', req_arg_bool,
-			1, NULL,
+			1, NULL, 0,
 			"enable/disable startup fencing");
 
 	set_opt_default(repeat_failed_fencing_ind,
 			"repeat_failed_fencing", '\0', req_arg_bool,
-			1, NULL,
+			1, NULL, 0,
 			"enable/disable retrying after fencing fails");
 
 	set_opt_default(enable_quorum_fencing_ind,
 			"enable_quorum_fencing", 'q', req_arg_bool,
-			1, NULL,
+			1, NULL, 0,
 			"enable/disable quorum requirement for fencing");
 
 	set_opt_default(enable_quorum_lockspace_ind,
 			"enable_quorum_lockspace", '\0', req_arg_bool,
-			1, NULL,
+			1, NULL, 0,
 			"enable/disable quorum requirement for lockspace operations");
 
+	set_opt_default(enable_helper_ind,
+			"enable_helper", '\0', req_arg_bool,
+			1, NULL, 0,
+			"enable/disable helper process for running commands");
+
 	set_opt_default(help_ind,
 			"help", 'h', no_arg,
-			-1, NULL,
+			-1, NULL, 0,
 			"print this help, then exit");
 
 	set_opt_default(version_ind,
 			"version", 'V', no_arg,
-			-1, NULL,
+			-1, NULL, 0,
 			"Print program version information, then exit");
 }
 
@@ -1521,6 +2033,9 @@ static void set_opt_cli(int argc, char **argv)
 		} else if (o->req_arg == req_arg_bool) {
 			o->cli_int = atoi(arg_str) ? 1 : 0;
 			o->use_int = o->cli_int;
+		} else if (o->req_arg == req_arg_uint) {
+			o->cli_uint = strtoul(arg_str, NULL, 0);
+			o->use_uint = o->cli_uint;
 		}
 	}
 
@@ -1588,12 +2103,17 @@ int main(int argc, char **argv)
 	set_opt_cli(argc, argv);
 	set_opt_file(0);
 
+	rv = node_config_init(CONF_FILE_PATH);
+	if (rv)
+		return 1;
+
 	strcpy(fence_all_device.name, "fence_all");
 	strcpy(fence_all_device.agent, "dlm_stonith");
 	fence_all_device.unfence = 0;
 
 	INIT_LIST_HEAD(&lockspaces);
 	INIT_LIST_HEAD(&fs_register_list);
+	INIT_LIST_HEAD(&run_ops);
 	init_daemon();
 
 	if (!opt(daemon_debug_ind) && !opt(foreground_ind)) {
@@ -1611,6 +2131,9 @@ int main(int argc, char **argv)
 
 	log_level(NULL, LOG_INFO, "dlm_controld %s started", RELEASE_VERSION);
 
+	if (opt(enable_helper_ind))
+		setup_helper();
+
 	memset(&act, 0, sizeof(act));
 	act.sa_handler = sigterm_handler;
 	rv = sigaction(SIGTERM, &act, NULL);
